import typing
from TransformsAI.Animo import ObjectInfo, GridObject, HoldableObject, VoxelGrid, Actor, GridTransform
from System import IEquatable_1, Array_1, Guid
from TransformsAI.Animo.Constants import TypeIds, Effects
from TransformsAI.Animo.Numerics import Vec2Int, Vec3Int
from System.Collections.Generic import List_1
from TransformsAI.Animo.Intents import Intent, UseIntent, HeldUseIntent
from TransformsAI.Animo.Rewards.Categories import RewardCategory
from TransformsAI.Animo.Rewards import ObjectDestroyedReward, Reward, TargetedReward, RewardTargetTypes, RewardTarget
from TransformsAI.Animo.Tools import ReadOnlyArray_1
from TransformsAI.Animo.Objects.Character import CharacterObject

class FlowerInfo(ObjectInfo, IEquatable_1[FlowerInfo]):
    def __init__(self) -> None: ...
    @property
    def CanStepOnObjects(self) -> bool: ...
    @property
    def CreatesObjects(self) -> Array_1[TypeIds]: ...
    @property
    def IsDestroyedInDeepWater(self) -> bool: ...
    @property
    def IsDestroyedInShallowWater(self) -> bool: ...
    @property
    def IsDestructible(self) -> bool: ...
    @property
    def IsGrabbable(self) -> bool: ...
    @property
    def IsMedium(self) -> bool: ...
    @property
    def IsOccupier(self) -> bool: ...
    @property
    def IsUsableOnGrid(self) -> bool: ...
    @property
    def MaximumBurnAmount(self) -> int: ...
    @property
    def TransformsInto(self) -> Array_1[TypeIds]: ...
    @property
    def Type(self) -> typing.Type[typing.Any]: ...
    @property
    def TypeId(self) -> TypeIds: ...
    @property
    def UsePriority(self) -> float: ...
    def CanReceiveEffect(self, effect: Effects) -> bool: ...
    def CreateObject(self) -> GridObject: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, left: FlowerInfo, right: FlowerInfo) -> bool: ...
    def __ne__(self, left: FlowerInfo, right: FlowerInfo) -> bool: ...
    def ToString(self) -> str: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: FlowerInfo) -> bool:...
        @typing.overload
        def __call__(self, other: ObjectInfo) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...



class FlowerObject(HoldableObject):
    def __init__(self) -> None: ...
    Info : ObjectInfo
    StepsBeforeFlowerWilts : int
    WiltDelay : int
    @property
    def BurnAmount(self) -> int: ...
    @property
    def Cell(self) -> Vec2Int: ...
    @property
    def Direction(self) -> Vec2Int: ...
    @property
    def GrabPriority(self) -> float: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def Holder(self) -> Actor: ...
    @Holder.setter
    def Holder(self, value: Actor) -> Actor: ...
    @property
    def InstanceId(self) -> Guid: ...
    @property
    def IntentsFiredThisTurn(self) -> List_1[Intent]: ...
    @property
    def IsBeingHeld(self) -> bool: ...
    @property
    def IsDestroyed(self) -> bool: ...
    @property
    def IsOnGrid(self) -> bool: ...
    @property
    def IsStoodUpon(self) -> bool: ...
    @property
    def MainIntent(self) -> Intent: ...
    @property
    def Position(self) -> Vec3Int: ...
    @property
    def PushedPosition(self) -> typing.Optional[Vec3Int]: ...
    @PushedPosition.setter
    def PushedPosition(self, value: typing.Optional[Vec3Int]) -> typing.Optional[Vec3Int]: ...
    @property
    def Transform(self) -> GridTransform: ...
    @property
    def TypeId(self) -> TypeIds: ...
    @property
    def UsedIntent(self) -> UseIntent: ...
    @property
    def VariantId(self) -> int: ...
    @VariantId.setter
    def VariantId(self, value: int) -> int: ...
    def GetUseIntentHeld(self, user: Actor) -> HeldUseIntent: ...
    def OnEffect(self, effect: Effects, source: GridObject, outIntents: List_1[Intent]) -> bool: ...


class FlowerRewards(RewardCategory):
    def __init__(self) -> None: ...
    DestroyFlower : ObjectDestroyedReward
    Gift : GiftFlowerReward
    MakeFlowerSeed : MakeFlowerSeedReward
    name : str
    Wilt : WiltFlowerReward
    @property
    def AssociatedObject(self) -> typing.Optional[TypeIds]: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def Rewards(self) -> ReadOnlyArray_1[Reward]: ...


class GiftFlowerReward(TargetedReward):
    def __init__(self, categoryId: str, rewardId: str) -> None: ...
    @property
    def CanUserTargetSelf(self) -> bool: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def HasTarget(self) -> bool: ...
    @property
    def RewardId(self) -> str: ...
    @property
    def Targets(self) -> Array_1[RewardTargetTypes]: ...
    def EvaluateOnIntentExecute(self, character: CharacterObject, intentCausedByCharacter: Intent, target: RewardTarget) -> bool: ...


class GiftIntent(HeldUseIntent):
    def __init__(self) -> None: ...
    Receiver : CharacterObject
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DidSucceed(self) -> bool: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> HoldableObject: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def User(self) -> Actor: ...
    @User.setter
    def User(self, value: Actor) -> Actor: ...


class MakeFlowerSeedIntent(HeldUseIntent):
    def __init__(self) -> None: ...
    @property
    def ChildIntents(self) -> List_1[Intent]: ...
    @property
    def DidSucceed(self) -> bool: ...
    @property
    def Grid(self) -> VoxelGrid: ...
    @property
    def IntentState(self) -> Intent.States: ...
    @property
    def Priority(self) -> float: ...
    @property
    def Source(self) -> FlowerObject: ...
    @property
    def Source(self) -> HoldableObject: ...
    @property
    def Source(self) -> GridObject: ...
    @property
    def User(self) -> Actor: ...
    @User.setter
    def User(self, value: Actor) -> Actor: ...


class MakeFlowerSeedReward(Reward):
    def __init__(self, categoryId: str, rewardId: str) -> None: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def HasTarget(self) -> bool: ...
    @property
    def RewardId(self) -> str: ...
    def EvaluateOnIntentExecute(self, character: CharacterObject, intentCausedByCharacter: Intent, target: typing.Optional[RewardTarget]) -> bool: ...


class WiltFlowerReward(Reward):
    def __init__(self, categoryId: str, rewardId: str) -> None: ...
    @property
    def CategoryId(self) -> str: ...
    @property
    def HasTarget(self) -> bool: ...
    @property
    def RewardId(self) -> str: ...
    def EvaluateOnIntentExecute(self, character: CharacterObject, intentCausedByCharacter: Intent, target: typing.Optional[RewardTarget]) -> bool: ...

